// Package declaration
package org.enableit.db ;

/*
 * PROJECT			: __PROJECT_NAME__
 *
 * COPYRIGHT        : Copyright (C) 2001 enableIT.org
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: DBUtils.java,v $
 * Revision 1.3  2001/12/03 12:42:31  __CVS_AUTHOR__
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.2  2001/04/24 08:15:08  __CVS_AUTHOR__
 * fix the null pointer exception in getNextIdBySQL
 *
 */

// Java Imports
import java.math.BigDecimal ; 
import java.sql.ResultSetMetaData ;
import java.rmi.RemoteException ;
import java.util.ArrayList ;
import java.util.HashMap ;
import java.util.Hashtable ;
import java.util.List ;
import java.util.Map ;
import java.util.TreeMap ;
import java.util.Iterator ;

// Log4J Imports
import org.apache.log4j.Category;

/**
 * Provides utilities to be used in conjunction with
 * the database proxies.
 * 
 * @version v1.1
 * Extracted the ResultSet conversion routines from the database proxies.
 * 
 * @author __AUTHOR__ 
 * 
 * @see org.enableit.db.DatabaseProxy DatabaseProxy<br/>
 * @see org.enableit.db.GenericDBProxy GenericDBProxy<br/>
 */
public class DBUtils
{

/*
 * Constructors
 */
	/**
	 * Default Constructor
	 */
	public DBUtils()
	{
		// No implementation
	}

/*
 * Methods
 */
	/**
	 * Converts a set of name-value pairs stored in a
	 * <CODE>java.util.Hashtable</CODE>
	 * to values only stored in a <CODE>java.util.ArrayList</CODE>
	 */
	public static ArrayList convertHashtableToArrayList( Hashtable attributes )
	{
		cat.debug("<ENTRY>");

		ArrayList al = new ArrayList( attributes.size() ) ;
		for ( Iterator i=attributes.keySet().iterator() ; i.hasNext() ; ) {
			// Beware of this, there may be a cast needed on i.next()
			al.add( attributes.get(i.next()) ) ;
		}


		cat.debug("<EXIT>");
		return al ;

	}

	/**
	 * Converts a set of name-value pairs stored in a
	 * <CODE>java.util.HashMap</CODE>
	 * to values only stored in a <CODE>java.util.ArrayList</CODE>
	 */
	public static ArrayList convertHashMapToArrayList( java.util.HashMap attributes )
	{
		cat.debug("<ENTRY>");

		ArrayList al = new ArrayList( attributes.size() ) ;
		for ( Iterator i=attributes.keySet().iterator() ; i.hasNext() ; ) {
			// Beware of this, there may be a cast needed on i.next()
			al.add( attributes.get(i.next()) ) ;
		}


		cat.debug("<EXIT>");
		return al ;

	}

    /**
     * Turns a <code>ResultSet</code> into an <code>ArrayList</code> 
     * of <code>TreeMaps</code>
	 * 
	 * @return list		
	 * An <CODE>ArrayList</CODE> of <CODE>TreeMap</CODE>s. Each
	 * map contains the column heading value pairs for a single result set
	 * row.
     */
    public static List convertResultToList(java.sql.ResultSet sqlResults) throws DBException
    {
		cat.debug("<ENTRY>");

        Map map   = null;
		List list = new ArrayList();
		StringBuffer sbuff=new StringBuffer();

		try {
			// get meta data
			ResultSetMetaData meta=sqlResults.getMetaData();
			int colCount=meta.getColumnCount();
			int rowCount=0;

			// get column headings
			String[] headings = new String[colCount];
        	for(int i = 0 ; i < colCount; i++) {
        		headings[i] = meta.getColumnLabel(i + 1); 
				if (headings[i] == null) {
					headings[i] = "column " + String.valueOf(i);
				}
        	}

			//Process all results in set
			while ( sqlResults.next() == true ) {

				map=new HashMap(colCount);

				// Processes a single row of the result set.
				// Each column becomes an element in the Map keyed
				// by the column heading
				for( int i = 0; i < colCount; i++) {

					// Get column value 
            		// Remember column numbers start from 1 !!!!
					Object resObj = sqlResults.getObject(i + 1);

					// Print out the retrieved info for verbose logging
					cat.debug("Column "  + String.valueOf(i + 1 ) + ": " + headings[i] + " : " 
							+ ((resObj== null) ? "null" : resObj.toString()));

					// Append value to string for SQL logging
                    sbuff.append(String.valueOf(resObj));
                    sbuff.append("|");

					// Store in map
					if(! sqlResults.wasNull()) {
                		if (resObj instanceof String) {
							map.put(headings[i], ((String)resObj).trim());
                   		} else {
                        	map.put(headings[i], resObj);
                    	}
                	}

					// send result trace to log
                    cat.info(sbuff.toString());
                    sbuff.setLength(0);
				}

				TreeMap treeMap=new TreeMap(map);
				list.add(treeMap);
			}
			
			// send result information to the log
			cat.info("<SQL_RESULT> Got results set with " + 
                    String.valueOf(list.size()) + " rows and " + 
                    String.valueOf(colCount) + " columns");

		} catch (Exception e) {
			cat.error(e.getMessage());
			throw new DBException(e.getMessage());
		}

		cat.debug("<EXIT>");
		return list;
    }

    /**
     * <p>Returns the next available id for the specified table</p>
     *
     * <p>The implementation makes use of a number fountain in the database
     * to avoid the chance of two EJB onstances attempting to use the same key</p>
     * 
     * @param tableName
     */
    public static int getNextIdUsingSQL( String tableName ) 
	throws RemoteException, DBException	    
    { 
		cat.debug("<ENTRY>");

        // Msg to report in the event of failure
        String msg = "Unable to find next id: " ; 

        int id = 0 ;

        try {
            // Update the next id to use first 
            try {
                int rowsAffected = GenericDBProxy.executeUpdate(SQL_SET_NEXT_ID + "'" + tableName + "'") ; 
                if (rowsAffected==0) {
                    // May be first time, try an insert then  
                    cat.info("STAGE 2: INIT NEXT ID") ; 
                    java.sql.Date now = new java.sql.Date(new java.util.Date().getTime()); 
                    String sql = SQL_INIT_ID + 
                                 "'" + now + "'," + 
                                 "'" + now + "'," + 
                                 "'" + tableName + "')" ; 
                    GenericDBProxy.executeUpdate(sql) ; 
                }
            } catch (Exception e) {
                // May be first time, try an insert then  
                java.sql.Date now = new java.sql.Date(new java.util.Date().getTime()); 
                String sql = SQL_INIT_ID + 
                             "'" + now + "'," + 
                             "'" + now + "'," + 
                             "'" + tableName + "')" ; 
                GenericDBProxy.executeUpdate(sql) ; 
            }

            // Now select the next id out 
            ArrayList results = (ArrayList)GenericDBProxy.executeQuery(
                    SQL_GET_NEXT_ID + "'" + tableName + "'", 
                    DatabaseProxy.LIST);
            cat.info(results.toString()) ; 
            if (results.size() > 0) {
                // Got my id
                TreeMap tm = (TreeMap)results.get(0);
                id = Integer.parseInt(tm.get("siNextId").toString()) ; 
            } else {
                // Log an exception.
                DBException dbe = new DBException(msg + "no id returned from select");
                cat.error( msg );
                throw dbe;
            }
            
	    } catch (DBException dbe) {	
			// Should already be logged at point raised, rethrow
			throw dbe ; 
	    } catch (Exception e) {
			/*
			 * Log an exception.
			 */
            e.printStackTrace(System.err) ; 
            cat.error(e.getClass().getName()) ; 
			DBException dbe = new DBException(msg + e.getMessage());
			cat.error( msg + e.getMessage() );
			throw dbe;
	    }

		cat.debug("<EXIT>");
        return id ; 
    } 

    /**
     * <p>Returns the next available id for the specified table</p>
     *
     * <p>The implementation makes use of a number fountain in the database
     * to avoid the chance of two EJB onstances attempting to use the same key</p>
     * 
     * @param tableName
     * 
     * @param spSupported
     * true - if stored procedures are to be used to execute the queries
     * false - to use dynamic SQL statements
     */
    public static int getNextId( String tableName, boolean spSupported ) 
	throws RemoteException, DBException	    
    { 
		cat.debug("<ENTRY>");

        // Msg to report in the event of failure
        String msg = "Unable to find next id" ; 

        if (!spSupported) {
            return getNextIdUsingSQL( tableName ) ; 
        }

        int id = 0 ; 
	    /*
	     * Single Result Set.
	     */
	    ArrayList parameterList = new ArrayList();
        parameterList.add( tableName ) ;
        try {
	    	ArrayList results = (ArrayList)GenericDBProxy.executeQueryProcedure(
                    PROC_GET_NEXT_ID, 
                    parameterList, DatabaseProxy.LIST);

			if (results.size() > 0) {
				// Store the reference for faster access.
				TreeMap tm = (TreeMap)results.get(0);
                cat.info(tm.toString()) ; 
                id = Integer.parseInt(tm.get("nextId").toString()) ; 
            } else {
				/*
				 * Log an exception.
				 */
				RemoteException re = new RemoteException();
				cat.error( msg );
				throw re;
			}

	    } catch (RemoteException re) {	
			// Should already be logged at point raised, rethrow
			throw re ; 
	    } catch (Exception e) {
			/*
			 * Log an exception.
			 */
			RemoteException re = new RemoteException(msg, e);
			cat.error( msg );
			throw re;
	    }

		cat.debug("<EXIT>");
        return id ; 
    } 

    /** 
     * <p>Adds a 'where' or 'and' clause as appropriate to join on the 
     * supplied column and value</p>
     * 
     * <p>If filterCol and / or filterVal are null then no filter is added. 
     * This could be modified to allow joins to null values, but presents 
     * issues of what datatype to send to the database</p>
     * 
     * @param sql 
     * The base query to which we wish to add a filter 
     * 
     * @param filterCol
     * The column to filter on 
     * 
     * @param filterVal 
     * The value to filter on 
     */
    public static String addFilter( String sql, String filterCol, String filterVal) 
    { 
		cat.debug("<ENTRY> sql: " + sql);

        if ( filterCol!=null && filterVal != null ) {
            // Determine whether filter value needs to be encased in quotes
            boolean numeric = true ; 
            for ( int i=0 ; i<filterVal.length() ; i++ ) {
                if ( !Character.isDigit(filterVal.charAt(i)) ) {
                    numeric = false ; 
                }
            }

            if (!numeric) {
                filterVal = "'" + filterVal + "'" ; 
            }

            // Determine whether WHERE or AND 
            if ((sql.toUpperCase().indexOf("WHERE"))!=-1) {
                // An AND clause required 
                sql += "\nAND " + filterCol + "=" + filterVal ; 
            } else { 
                // A WHERE clause required
                sql += "\nWHERE " + filterCol + "=" + filterVal ; 
            } 
        }
        
		cat.debug("<EXIT>");
        return sql ; 
    } 

    /** 
     * <p>Adds a 'where' or 'and' clause as appropriate to join on the 
     * supplied column and value</p>
     * 
     * <p>If filter name and / or filter value are null then no filter is added. 
     * This could be modified to allow joins to null values, but presents 
     * issues of what datatype to send to the database</p>
     * 
     * @param sql 
     * The base query to which we wish to add a filter 
     * 
     * @param filter
     * All the encapsulated filter data 
     */
    public static String addFilter( String sql, DBFilter filter ) 
    { 
		cat.debug("<ENTRY> sql: " + sql);

        if ( filter.getName()!=null && filter.getValue() != null ) {
            // Determine whether filter value needs to be encased in quotes
            boolean numeric = true ; 
            for ( int i=0 ; i<filter.getValue().length() ; i++ ) {
                if ( !Character.isDigit(filter.getValue().charAt(i)) ) {
                    numeric = false ; 
                }
            }

            if (!numeric) {
                filter.setValue("'" + filter.getValue() + "'") ; 
            }

            // Log the clause to be added
            cat.info("Adding filter: " + filter) ; 

            // Determine whether WHERE or AND 
            if ((sql.toUpperCase().indexOf("WHERE"))!=-1) {
                // An AND clause required 
                sql += "\nAND " + filter.getName() + filter.getMatch() + filter.getValue() ; 
            } else { 
                // A WHERE clause required
                sql += "\nWHERE " + filter.getName() + filter.getMatch() + filter.getValue() ; 
            } 
        }
        
		cat.debug("<EXIT>");
        return sql ; 
    } 

    /** 
     * <p>Adds a 'where' or 'and' clause as appropriate to join on the 
     * supplied column and value</p>
     * 
     * <p>If filter name and / or filter value are null then no filter is added. 
     * This could be modified to allow joins to null values, but presents 
     * issues of what datatype to send to the database</p>
     * 
     * @param sql 
     * The base query to which we wish to add a filter 
     * 
     * @param filter
     * A java.util.List of DBFilter objects containing the encapsulated filter data 
     */
    public static String addFilters( String sql, List filters ) 
    { 
		cat.debug("<ENTRY> sql: " + sql);

        for ( Iterator i=filters.iterator() ; i.hasNext() ; ) {
            sql = addFilter(sql, (DBFilter)i.next()) ; 
        }

		cat.debug("<EXIT>");
        return sql ; 
    } 

/*
 * Properties
 */

    /** 
     * The Log4J <code>Category</code> doing the logging.
     * Same <code>Category</code> is used throughout the package.
     */
    protected static Category cat = Category.getInstance("org.enableit.db");
	
    /** 
	 * Name of stored procedure used for getNextId
	 */
	private static final String PROC_GET_NEXT_ID = "GetNextId" ;

	/** 
	 * Dynamic SQL used for getNextIdUsingSQL to update the id to the next one
	 */
	private static final String SQL_SET_NEXT_ID = "UPDATE SystemIdentity SET siNextId = siNextId + 1 WHERE siName=" ;

    /** 
	 * Dynamic SQL used for getNextId
	 */
	private static final String SQL_GET_NEXT_ID = "SELECT siNextId FROM SystemIdentity WHERE siName=" ;
	
    /** 
	 * Dynamic SQL used for getNextId, initialises the table if no previous query 
	 */
	private static final String SQL_INIT_ID = "INSERT INTO SystemIdentity (" + 
								              "siNextId	        ," +    
                                              "siDesc			," +    
                                              "siCreationUser	," +    
                                              "siUpdateUser	    ," +    
                                              "siCreationDate	," +    
                                              "siUpdateDate	    ," +    
                                              "siName           )" +  
                                              "VALUES (          " + 
                                              "1				    ," + 
                                              "'description here'	," + 
                                              "'sysuser'	        ," + 
                                              "'sysuser'	        ," ;  

    /** 
     * Information on the exact CVS version accessible after compilation
     */
    public static final String about = "$Revision: 1.3 $";

}

