// Package declaration 
package org.enableit.db ; 

/*
 * PROJECT			: __PROJECT_NAME__
 *
 * COPYRIGHT        : Copyright (C) 2000 enableIT.org
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log$
 */

// Java imports 
import java.io.File ; 
import java.io.FileReader ; 
import java.io.LineNumberReader ; 
import java.util.ArrayList ; 
import java.util.Iterator ; 
import java.util.StringTokenizer ; 

// Log4J Imports
import org.apache.log4j.Category;

/**  
 * A class to read CSV files and load data into a database. 
 * 
 * @version v1.3
 * 
 * @author __AUTHOR__ 
 * 
 */
public class DataLoader 
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	public DataLoader() 
	{

	}

    /** 
	 * Constructor setting mandatory properties. 
	 */
	public DataLoader(String file) 
	{
        setFile(file); 
	}

/*
 * Set Methods
 */

    /**  
     * Set the file to be loaded.
     */
    public void setFile(File file) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        
        this.file = file ; 
    } 

    /**  
     * Set the file to be loaded.
     */
    public void setFile(String file) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        
        this.file = new File(file) ; 
    } 

    /**  
     * Set the file to be loaded. <br>
     * This field is not mandatory if metadat is supplied at the top of the 
     * CSV file. 
     * 
     */
    public void setTable(String table) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        
        this.table = table ; 

        // Log the exit from the method.
        cat.debug("<EXIT>") ;
        
    } 

    /**  
     * Set the names of columns to be inserted into.
     * 
     * @param 
     * columns as a comma separated list.
     */
    public void setColumnNames(String columns) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        cat.debug("... with param: columns=" + columns);
        
        StringTokenizer st = new StringTokenizer(columns, ",") ; 
        this.columns = new ArrayList() ; 

        while (st.hasMoreElements()) {
            this.columns.add(new Column(st.nextToken())) ;         
        }
    } 

    /**  
     * Set the datatypes of the columns to be inserted into.
     * 
     * @param 
     * columns as a comma separated list.
     */
    public void setColumnDatatypes(String datatypes) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        cat.debug("... with param: columns=" + datatypes);
        
        StringTokenizer st = new StringTokenizer(datatypes, ",") ; 

        for ( Iterator i=columns.iterator() ; i.hasNext() ; ) {
            Object obj = i.next() ; 
            Column col = (Column)obj ; 
            col.setDatatype(st.nextToken()) ; 
        }

        // Log the exit from the method.
        cat.debug("<EXIT>") ;
        cat.debug("... columns=" + columns);        
    } 

    /**  
     * Set the datasource name to get connections from.
     */
    public void setDataSourceName(String dataSourceName) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        
        this.dataSourceName = dataSourceName ; 
    } 

/*  
 * Implementation methods
 */

    /**  
     * Do the load.
     */
    public void execute() 
    throws java.io.IOException, java.sql.SQLException 
    {   
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
    
        // Open file 
        FileReader fileReader = new FileReader(file) ;         
        LineNumberReader reader = new LineNumberReader(fileReader) ;

        // Read and process each row 
        String line = reader.readLine() ;  ; 
        do {

            if (line.startsWith("#")) {
                // This is a comment, perhaps containing meta-data
                processComment(line) ; 
            } else { 
                // presumable a data row 
                process(line) ; 
            } 

            line = reader.readLine() ; 
        } while (line!=null);

    } 
    
    /**  
     * Processes a comment line, looking for meta data. 
     */
    private void processComment(String line)
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
        
        // Look for the table name
        if (line.startsWith("#TABLE ")) {
            setTable(line.substring(7)) ; 
        }
        if (line.startsWith("#NAMES ")) {
            setColumnNames(line.substring(7)) ; 
        }
        if (line.startsWith("#DATATYPES ")) {
            setColumnDatatypes(line.substring(12)) ; 
        }
    
    } 

    /**  
     * Processes a data line. 
     */
    private void process(String line) 
    { 
        // Log the entry into the method.
        cat.debug("<ENTRY>") ;
      
        StringTokenizer values = new StringTokenizer(line, ",") ; 
        if (values.countTokens()!=columns.size()) {
            // erroroneous number of values, skip 
            cat.warn("This line has the worng number of values:\n" + line) ; 
        }

        // Prepare the SQL 
        StringBuffer sb = new StringBuffer() ; 
        sb.append("INSERT INTO ") ; 
        sb.append(table) ; 
        sb.append("(") ; 
        for ( Iterator i=columns.iterator() ; i.hasNext() ; ) {
            sb.append( ((Column)i.next()).getName() ) ; 
            if (i.hasNext()) {
                sb.append(",") ; 
            } 
        }
        sb.append(")") ; 
        sb.append("VALUES (") ; 

        for ( Iterator i=columns.iterator() ; i.hasNext() ; ) {
            String datatype = ((Column)i.next()).getDatatype() ; 
            if (datatype.equals("DATED") || datatype.startsWith("CHAR")) {
                sb.append("'" + values.nextToken() + "'") ; 
            } else { 
                sb.append(values.nextToken()) ; 
            } 
            if (i.hasNext()) {
                sb.append(",") ; 
            } 
        }
        sb.append(")") ; 
      
        // Execute the SQL 
        try {
            GenericDBProxy.executeUpdate(sb.toString()) ;         	
        } catch (Exception e) {
            cat.error(e) ;     
        }
    } 

    /**  
     * Used as a simple test of the class. 
     */
    public static void main (String args[]) 
    { 
        try {
            new DataLoader(args[0]).execute() ;         	
        } catch (Exception e) {
            e.printStackTrace() ; 
        }
    } 

/*
 * Properties
 */

    /** 
     * The Log4J <code>Category</code> doing the logging.
     * Same <code>Category</code> is used throughout the package.
     */
    protected static Category cat = Category.getInstance("org.eableit.db");

    /**  
     * The file to read. 
     */
    private File file ; 

    /**  
     * The Table to load into.
     */
    private String table ; 

    /**  
     * The columns to load into.
     */
    private ArrayList columns ; 

    /**  
     * The datasource name to use to get a database connection.
     */
    private String dataSourceName ; 
    
	/**  
	 * CVS info about this class and its current version
	 */
	public static final String about = "$Id$" ;

}
