    
package org.enableit.db ;

/*
 * PROJECT			: __PROJECT_NAME__
 *
 * COPYRIGHT        : Copyright (C) 2000 enableIT.org
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: GenericDBProxy.java,v $
 * Revision 1.2  2001/12/03 12:42:31  __CVS_AUTHOR__
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.1.1.1  2001/04/23 05:27:12  __CVS_AUTHOR__
 * finally get under source control
 *
 */

// Java Imports
import java.util.*;
import java.sql.*;

// Log4J Imports
import org.apache.log4j.Category;

/**
 * <p>Proxy class to wrap any java application database access in a simple form</p>
 *
 * @version v1.0
 * 
 * @author __AUTHOR__ 
 * 
 */
public class GenericDBProxy extends DatabaseProxy
{

    /**
     * Execute the supplied SQL update query against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sql
	 * The SQL update to execute
	 *
	 * @return int
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(String sql) throws DBException
    {
        cat.debug("<ENTRY>") ; 

        int rowsAffected    = 0;

        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
			rowsAffected=executeUpdate(conn, sql);
		} catch (DBException dbE){
			cat.error(dbE.getMessage());
			throw dbE;
		} catch (Exception e) {
			cat.error(e.getMessage());
			throw new DBException(e.getMessage());
		} finally {
		}

        cat.debug("<EXIT>") ; 
		return rowsAffected;
    }

    /**
     * Execute the supplied SQL query against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sql
	 * The SQL query to execute
	 *
	 * @param returnType
	 * int defining the return object types, enumerated on DatabaseProxy
	 *
	 * @return Object
	 * The SQL results as the object type requested
	 *
	 * @throws DBException
	 * If execution of the query failed.
     */
    public static Object executeQuery(String sql, int returnType) 
    throws DBException
    {
        cat.debug("<ENTRY>: sql: " + sql);

		Object result     = null;

        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
            result=executeQuery(conn, sql, returnType);
        	conn.close() ; 
		} catch (DBException dbe) {
            // already logged, just rethrow
			throw dbe ;
		} catch (Exception e) {
			cat.error(e.getMessage());
			throw new DBException("ERROR: " + e.getMessage());
		} finally {

        }

        cat.debug("<EXIT>") ; 		
        return result;
    }

    /**
     * Execute the supplied stored procedure against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sp
	 * A string containing the stored procedure name
	 *
     * @param parms
     * <code>ArrayList</code> holding parameters required for stored proc
	 *
	 * @return int
	 * The number of rows affected by the stored procedure executed
	 *
	 * @throws DBException
	 * If execution of the query failed
     */
    public static int executeDmlProcedure(String sp, ArrayList parms) 
    throws DBException
    {
        cat.debug("<ENTRY>: sp, parms: " + sp + "," + parms.toString());

		int i = 0;

        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
			executeDmlProcedure(conn, sp, parms);
            conn.close() ;
		} catch (DBException dbE){
			cat.error(dbE.getMessage());
            throw dbE;
		} catch (Exception e) {
			cat.error(e.getMessage());
			throw new DBException(e.getMessage());
		} finally {
		}

        cat.debug("<EXIT>") ; 
		return i ;

	}

	/**
     * Execute the supplied stored procedure against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sp
	 * A string containing the stored procedure name
	 *
     * @param parms
     * <code>ArrayList</code> holding parameters required for stored proc
	 *
	 * @param returnType
	 * int defining the return object types, enumerated on DatabaseProxy
	 *
	 * @return object
	 * <code>java.lang.Object</code> containing any rows found
     * Object type matches the enumeration in <code>DatabaseProxy</code>
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static java.lang.Object executeQueryProcedure(String sp, List parms, int returnType) 
    throws DBException
    {
        cat.debug("<ENTRY> sp, parms: " + sp + "," + parms.toString());

        Object o = null ; 
        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
    		o = executeQueryProcedure(conn, sp, parms, returnType) ;
		} catch (DBException dbE){
			cat.error(dbE.getMessage());
			throw dbE;
		} catch (Exception e) {
			cat.error(e.getMessage());
			throw new DBException(e.getMessage());
		} finally {
        }

        cat.debug("<EXIT>") ; 
		return o ;
	}


    /** 
     * The Log4J <code>Category</code> doing the logging.
     * Same <code>Category</code> is used throughout the package.
     */
    protected static Category cat = Category.getInstance("org.enableit.db");

    /** 
     * Information on the exact CVS version accessible after compilation
     */
    public static final String about = "$Revision: 1.2 $";

}


