// Package declaration 
package org.enableit.db.test ; 

// Java imports 
import java.sql.Connection ; 
import java.util.List ; 

// JUnit Imports
import junit.framework.*;

// dbtools library 
import org.enableit.db.* ; 

/** 
 * A test class to exercise the DatabaseProxy
 */
public class DatabaseProxyTest extends TestCase
{

/*
 * Constructors
 */

    public DatabaseProxyTest(String theName)
    {
        super(theName);
    }



/*
 * Methods
 */

    /** 
     * The method provided for by the JUnit framework in order that common setup 
     * tasks are completed before tests run
     * 
     */
    protected void setUp() 
    throws Exception 
    { 
        try {
            conn = ConnectionFactory.getConnection() ; 
            assertNotNull(conn) ; 
        } catch (Exception e) {
            fail(e.getMessage()) ; 
        }
    } 

    /** 
     * The method provided for by the JUnit framework in order that common setup 
     * tasks are cleaned up after tests are run
     * 
     */
    protected void tearDown() 
    throws Exception 
    { 
        try {
            conn.close() ; 
            conn = null ; 
        } catch (Exception e) {
            fail(e.getMessage()) ; 
        }
    } 

    /** 
     * 
     */
    public void executeQuery() 
    { 
        try {
            List results = DatabaseProxy.executeQuery( conn, "SELECT * FROM Task" ) ;         
            assertNotNull(results) ; 
        } catch (Exception e) {
            e.printStackTrace() ; 
            fail(e.getMessage()) ; 
        }
    } 

    /** 
     * Invoke the tests 
     */
    public static Test suite() {
        TestSuite suite= new TestSuite();
        suite.addTest(new DatabaseProxyTest("executeQuery"));
        return suite;
    }

/*
 * Properties
 */

    /** Connection used to perform the tests */
    private Connection conn ; 
}
