// Package declaration 
package org.thestephensons.db; 

// Java imports
import java.lang.reflect.Constructor;
import java.sql.Connection ; 
import java.sql.SQLException ; 
import java.util.Hashtable ; 
import java.util.Iterator ; 

// Extension imports 
import javax.management.* ; 

// My imports 
import org.enableit.util.Log ; 
import org.enableit.util.PersistenceManager ; 
import org.enableit.util.PersistenceManagerFactory ; 

/*
 * PROJECT			: 
 *
 * COPYRIGHT        : Copyright Tim Stephenson (C) 2001
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: ConnectionFactoryMBean.java,v $
 * Revision 1.1  2001/12/03 12:44:18  tstephen
 * allow for JMX management of ConnectionFactory class
 *
 */
 
/** 
 * Provides JMX instrumentation for the <code>ConnectionFactory</code>
 * 
 * It is necessary to instrument the <code>ConnectionFactory</code> 
 * in a separate class because the JMX spec requires a public constructor 
 * and the <code>ConnectionFactory</code> is a singleton class
 */
public class ConnectionFactoryMBean extends java.lang.Object 
implements DynamicMBean, PersistentMBean
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	public ConnectionFactoryMBean() 
	{
        // Initialise the meta data object: attributesInfo
       	attributesInfo[0] = new MBeanAttributeInfo("DRIVER",
                "java.lang.String",
                "Driver: name of the JBDC type 4 driver class",
                true,
                true,
                false);
        attributesInfo[1] = new MBeanAttributeInfo("PROTOCOL",
                "java.lang.String",
                "Protocol: The communication protocol used to connect to the database.",
                true,
                true,
                false);
        attributesInfo[2] = new MBeanAttributeInfo("SERVER_NAME",
                "java.lang.String",
                "Server Name: the name or IP address of the dataserver host",
                true,
                true,
                false);
        attributesInfo[3] = new MBeanAttributeInfo("DATABASE_NAME",
                "java.lang.String",
                "Database Name: The name of the database.",
                true,
                true,
                false);
        attributesInfo[4] = new MBeanAttributeInfo("PORT",
                "java.lang.Integer",
                "Port: the port on which the dataserver is listening.",
                true,
                true,
                false);
        attributesInfo[5] = new MBeanAttributeInfo("USER",
                "java.lang.String",
                "User: the username with which to connect to the dataserver.",
                false,
                true,
                false);
        attributesInfo[6] = new MBeanAttributeInfo("PASSWORD",
                "java.lang.String",
                "Password: The user's password to connect to the dataserver.",
                false,
                true,
                false);
        
        // Initialise the meta data object: constructors
        Constructor[] constructors = this.getClass().getConstructors();
        constructorsInfo[0] = new MBeanConstructorInfo("ConnectionFactoryMBean(): Constructs a ConnectionFactoryMBean object",
                constructors[0]);
        
	    MBeanParameterInfo[] params = null;        
        operationsInfo = new MBeanOperationInfo[3] ; 
        operationsInfo[0] = new MBeanOperationInfo (
                "testConnection" , 
                "Creates and releases a connection to the database with the provided parameters" , 
                params , 
                "java.lang.Boolean" , 
                MBeanOperationInfo.INFO ) ;
        operationsInfo[1] = new MBeanOperationInfo (
                "store" , 
                "Stores the MBean's properties via the PersistenceManager" , 
                params , 
                "void" , 
                MBeanOperationInfo.ACTION ) ;
        operationsInfo[2] = new MBeanOperationInfo (
                "load" , 
                "Loads the MBean's properties via the PersistenceManager" , 
                params , 
                "Hashtable" , 
                MBeanOperationInfo.ACTION ) ;
        
        mBeanInfo = new MBeanInfo(dClassName,
                descriptionInfo,
                attributesInfo,
                constructorsInfo,
                operationsInfo,
                new MBeanNotificationInfo[0]);

	}

/*
 * DynamicMBean Methods
 */
	/** 
	 * Returns MBean meta-data
	 */
	public MBeanInfo getMBeanInfo() 
	{ 
        // return the information we want to expose for management:
        // the mBeanInfo private field has been built at instanciation time,
        return(mBeanInfo);
    } 

	/** 
	 * Returns requested attribute
	 */
	public Object getAttribute(String name)    
    throws AttributeNotFoundException,
           MBeanException,
           ReflectionException
    { 

        Object value = null ; 
        if ( name.equals("DRIVER") ) {
            value = driver ; 
        } else if ( name.equals("PROTOCOL") ) {
            value = protocol ; 
        } else if ( name.equals("SERVER_NAME") ) {
            value = serverName ; 
        } else if ( name.equals("DATABASE_NAME") ) {
            value = databaseName ;
        } else if ( name.equals("PORT") ) {
            value = port ; 
        } else if ( name.equals("USER") ) {
            value = user ; 
        } else if ( name.equals("PASSWORD") ) {
            value = password ; 
        } else { 
            throw new AttributeNotFoundException() ; 
        } 

        return value ; 
	} 

	/** 
	 * Set an attribute on the ConnectionFactory
	 */
	public void setAttribute(Attribute attribute) 
    throws AttributeNotFoundException,
           InvalidAttributeValueException,
           MBeanException,
           ReflectionException
    {
        String name = attribute.getName();
        Object value = attribute.getValue();
        System.out.println("Entry: setAttribute: name, value: " + name + "," + value) ;

        // Look for recognised attributes 
        if ( name.equals("DRIVER") ) {
            driver = (String)value ; 
            ConnectionFactory.getInstance().setProperty(name, driver); 
        } else if ( name.equals("PROTOCOL") ) {
            protocol = (String)value ; 
            ConnectionFactory.getInstance().setProperty(name, protocol); 
        } else if ( name.equals("SERVER_NAME") ) {
            serverName = (String)serverName ; 
            ConnectionFactory.getInstance().setProperty(name, serverName); 
        } else if ( name.equals("DATABASE_NAME") ) {
            databaseName = (String)value ; 
            ConnectionFactory.getInstance().setProperty(name, databaseName); 
        } else if ( name.equals("PORT") ) {
            port = (Integer)value ; 
            ConnectionFactory.getInstance().setProperty(name, port.toString()); 
        } else if ( name.equals("USER") ) {
            user = (String)value ; 
            ConnectionFactory.getInstance().setProperty(name, user); 
        } else if ( name.equals("PASSWORD") ) {
            password = (String)value ; 
            ConnectionFactory.getInstance().setProperty(name, password); 
        } else { 
            throw new AttributeNotFoundException() ; 
        } 

    }

	/** 
	 * Returns requested attributes 
	 */
	public AttributeList getAttributes(String[] attrNames) 
	{
        AttributeList al = new AttributeList() ; 
        Object value = null ; 

        for ( int i=0 ; i<attrNames.length ; i++ ) {
            try {
                System.out.println(attrNames[i]) ; 
            	value = getAttribute(attrNames[i]) ; 
                al.add( new Attribute(attrNames[i], value) ) ; 
            } catch (AttributeNotFoundException anfe) {
                anfe.printStackTrace() ; 
                // No need to do anything, just continue
            } catch (MBeanException mbe) {
                mbe.printStackTrace() ; 
                // No need to do anything, just continue
            } catch (ReflectionException re) {
                re.printStackTrace() ; 
                // No need to do anything, just continue
            }
        }

        return al ; 
	}

	/** 
	 * Sets more than one attribute on the ConnectionFactory
	 */
	public AttributeList setAttributes(AttributeList inAttrs) 
	{
        AttributeList setAttrs = new AttributeList() ; 

        try {
            Attribute attr = null ; 
            for (Iterator i=inAttrs.iterator() ; i.hasNext() ; ) {
                attr = (Attribute)i.next() ; 
                setAttribute( attr ) ; 
                setAttrs.add( attr ) ;
            }
        	
        } catch (AttributeNotFoundException anfe) {
            Log.write(anfe) ; 
        } catch (InvalidAttributeValueException iave) {
            Log.write(iave) ; 
        } catch (MBeanException mbe) {
            Log.write(mbe) ; 
        } catch (ReflectionException re) {
            Log.write(re) ; 
        }
        return setAttrs ; 
	}

	/** 
	 * Invoke an MBean operation
	 */
	public Object invoke ( String actionName, 
                           Object[] params, 
						   String[] signature)
    throws MBeanException, ReflectionException
    { 
        Object retObj = null ; 
        System.out.println("enterign invoke") ; 
        if (actionName.equals("testConnection")) {
            System.out.println("enterign invoke: testConnection") ; 
            try {
                // verify signature 

                // invocation 
                ConnectionFactory cf = ConnectionFactory.getInstance() ; 
                System.out.println("b4 connection open ") ; 
                Connection conn = cf.getConnection() ; 
                System.out.println("b4 connection close ") ; 
                conn.close() ; 
                retObj = Boolean.TRUE ; 
            } catch (SQLException sqle) { 
                retObj = Boolean.FALSE ;
                sqle.printStackTrace() ; 
            } catch (Exception e) { 
                retObj = Boolean.FALSE ;
                e.printStackTrace() ; 
            } 
        } else if (actionName.equals("store")) {
            try {
            	this.store() ; 
            } catch (Exception e) {
                e.printStackTrace() ; 
            }
        } else if (actionName.equals("load")) {
            try {
            	this.load() ; 
            } catch (Exception e) {
                e.printStackTrace() ; 
            }
        } else {
            throw new ReflectionException(null, "No such method") ; 
        } 
    
        return retObj ; 
    } 

/* 
 * Persistent MBean methods
 */
    /** 
     * Stores the properties of the MBean by emplopying the persistence manager
     */
    public void store()
    throws MBeanException, RuntimeOperationsException, InstanceNotFoundException    
    { 
        Log.write("ENTRY", true, Log.METHOD_ENTRY) ; 
        
        Hashtable keys = new Hashtable() ; 
        keys.put("type", getClass().getName()) ; 
        // Store only the transient data, rather than the entire class for simplicity 
        Hashtable attributes = new Hashtable() ;
        attributes.put("driver", driver) ; 
        attributes.put("protocol", protocol) ; 
        attributes.put("serverName", serverName) ; 
        attributes.put("databaseName", databaseName) ; 
        attributes.put("port", port) ; 
        attributes.put("user", user) ; 
        attributes.put("password", password) ; 

        // Get the persistence manager to use 
        PersistenceManager pm = PersistenceManagerFactory.getInstance() ; 
        pm.store("domain", keys, attributes) ;

        Log.write("EXIT", true, Log.METHOD_EXIT) ; 
    } 

    /** 
     * Loads the properties of the MBean by emplopying the persistence manager
     */
    public void load()
    throws MBeanException, RuntimeOperationsException, InstanceNotFoundException    
    { 
        Log.write("ENTRY", true, Log.METHOD_ENTRY) ; 
        
        try {
            // Get the persistence manager to use 
            PersistenceManager pm = PersistenceManagerFactory.getInstance() ; 

            // Load only the transient data, rather than the entire class for simplicity 
            Hashtable keys = new Hashtable() ; 
            keys.put("type", getClass().getName()) ; 
            Hashtable attributes = pm.load("domain", keys) ;

            // Extract and tell the connection factory of the newly read properties
            setAttribute( new Attribute("driver", (String)attributes.get("driver")) ) ; 
            setAttribute( new Attribute("protocol", (String)attributes.get("protocol")) ) ; 
            setAttribute( new Attribute("serverName", (String)attributes.get("serverName")) ) ;         
            setAttribute( new Attribute("databaseName", (String)attributes.get("databaseName")) ) ; 
            setAttribute( new Attribute("port", new Integer(attributes.get("port").toString())) ) ; 
            setAttribute( new Attribute("user", (String)attributes.get("user")) ) ; 
            setAttribute( new Attribute("pasword", (String)attributes.get("password")) ) ; 
        	
        } catch (AttributeNotFoundException anfe) {
            Log.write(anfe) ; 
            throw new MBeanException(anfe) ; 
        } catch (InvalidAttributeValueException iave) {
            Log.write(iave) ; 
            throw new MBeanException(iave) ; 
        } catch (ReflectionException re) {
            Log.write(re) ; 
            throw new MBeanException(re) ; 
        }        
        
        Log.write("EXIT", true, Log.METHOD_EXIT) ; 
    } 

/*
 * Properties
 */

    private String	driver = "com.sybase.jdbc2.jdbc.SybDriver" ;
    private String	protocol = "Tds" ;
    private String	serverName = "localhost";
    private String	databaseName = "testDB";
    private Integer	port = new Integer(2638) ;
    private String	user = "dba";
    private String	password = "sql";

    private String dClassName = this.getClass().getName();
    private String descriptionInfo = "MBean for managing the ConnectionFactory";

    private MBeanAttributeInfo[] attributesInfo = new MBeanAttributeInfo[7];
    private MBeanConstructorInfo[] constructorsInfo = new MBeanConstructorInfo[1];
    private MBeanOperationInfo[] operationsInfo = new MBeanOperationInfo[0];
    private MBeanInfo mBeanInfo = null;
  
	/**  
	 * CVS info about this class and its current version
	 */
	public static final String about = "$Id: ConnectionFactoryMBean.java,v 1.1 2001/12/03 12:44:18 tstephen Exp $" ;

}
