// Package declaration 
package org.thestephensons.db ; 

/*
 * PROJECT			: 
 *
 * COPYRIGHT        : Copyright Tim Stephenson (C) 2001
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: DBFilter.java,v $
 * Revision 1.1  2001/12/03 12:45:27  tstephen
 * encapsulate the functionality of a SQL WHERE clause
 *
 */
 
// Imports

public class DBFilter 
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	public DBFilter()
	{

	}

	/** 
	 * Convenience Constructor setting name and value as object created 
	 */
	public DBFilter(String name, String value)
	{
        this.name = name ; 
        this.value = value ; 
	}

	/** 
	 * Convenience constructor setting name, value and match as object created 
	 */
	public DBFilter(String name, String value, int match)
	{
        this.name = name ; 
        this.value = value ; 
        this.match = match ; 
	}

/*
 * Setter Methods
 */
    /** 
     * The column to filter on 
     */
    public void setName(String name)
    { 
        this.name = name ; 
    }
    
    /** 
     * The value of column to filter on 
     */
    public void setValue(String value)
    { 
        this.value = value ; 
    }

    /** 
     * The type of match required
     */
    public void setMatch(int match)
    { 
        this.match = match ; 
    }

/*
 * Getter Methods
 */
    /** 
     * Returns the column to filter on 
     */
    public String getName()
    { 
        return name ; 
    }
    
    /** 
     * Returns the value of column to filter on 
     */
    public String getValue()
    { 
        return value ; 
    }

    /** 
     * Returns the type of match required as an int 
     * 
     * @return matchType 
     * one of the matches enumerated in this class as XXX_MATCH properties
     */
    public int getMatchType()
    { 
        return match ; 
    }

    /**  
     * Return a string representaion of the match 
     */
    public String getMatch() 
    { 
        // Determine the match as a string 
        String match = "=" ; 
        switch (this.match) {
        case DBFilter.EXACT_MATCH :
            match = "=" ; 
            break ; 
        case DBFilter.LESS_THAN_MATCH :
            match = "<=" ; 
            break ; 
        case DBFilter.GREATER_THAN_MATCH :
            match = ">=" ; 
            break ;             
        }

        return match ; 
    } 


    /** 
     * Returns a string representation of the filter 
     */
    public String toString() 
    { 
        return name + getMatch() + value ; 
    } 
/*
 * Properties
 */

    /** 
     * The column to filter on 
     */
    private String name ; 

    /** 
     * The value of column to filter on 
     */
    private String value ; 

    /** 
     * The type of match required 
     */
    private int match ; 

    /** A valid match type */
    public static final int EXACT_MATCH = 0 ; 

    /** A valid match type */
    public static final int LESS_THAN_MATCH = 1 ; 

    /** A valid match type */
    public static final int GREATER_THAN_MATCH = 2 ; 

    /**  
	 * CVS info about this class and its current version
	 */
	public static final String about = "$Id: DBFilter.java,v 1.1 2001/12/03 12:45:27 tstephen Exp $" ;

}
