// Package declaration 
package org.thestephensons.db ; 

/*
 * PROJECT			: 
 *
 * COPYRIGHT        : Copyright Tim Stephenson (C) 2001
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: DBPersistenceManager.java,v $
 * Revision 1.1  2001/12/03 12:45:58  tstephen
 * Initial Creation
 *
 */
 
// Java imports 
import java.sql.* ; 
import java.util.Hashtable ; 
import java.util.List ; 
import java.util.TreeMap ; 

// My Imports
import org.enableit.util.Log ; 
import org.enableit.util.PersistenceManager ; 

public class DBPersistenceManager implements PersistenceManager
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	public DBPersistenceManager() 
	{

	}

/*
 * PersistenceManager methods
 */
    /**  
     * Stores the state of the object received, keyed by the domain and keys table
     */
    public void store(String domain, Hashtable keys, Object obj) 
    { 
        Log.write("ENTRY", true, Log.METHOD_ENTRY) ; 

        try {
            Connection conn = ConnectionFactory.getConnection() ; 

            // It should be impossible to store two versions of the same MBean, so delete any existing
            String sql = "DELETE FROM MBean WHERE mbeanName = '" + domain + ":" + keys.toString() + "'" ;
            DatabaseProxy.executeUpdate(conn, sql) ; 

            // Need an id 
            int id = DBUtils.getNextIdUsingSQL("MBean") ; 

            sql = "insert MBean (mbeanId, mbeanName, mbeanObject) values( ?, ?, ? )" ; 
            java.sql.PreparedStatement ps = conn.prepareStatement( sql );
            ps.setInt( 1, id );
            ps.setObject( 2, domain + ":" + keys.toString() );
            ps.setObject( 3, (Object)obj );
            
            Log.write("SQL_VERBOSE executing: " + ps.toString(), true, Log.SQL_VERBOSE) ; 

            // Do it 
            ps.executeUpdate();

  			// Print any warnings chained to statement object
			SQLWarning warning = ps.getWarnings();
			while( warning != null ){
				Log.write("<SQL Warning :>" + warning.toString(), true);
				warning = warning.getNextWarning() ;
			}
          
            // Cleanup 
            conn.close() ; 
            conn = null ; 

        } catch (Exception e) {
            e.printStackTrace() ; 
            Log.write(e) ; 
        }
        Log.write("EXIT", true, Log.METHOD_EXIT) ; 
    } 

    /**  
     * Loads the state of the object received, keyed by the domain and keys table
     */
    public Hashtable load(String domain, Hashtable keys) 
    { 
        Log.write("ENTRY", true, Log.METHOD_ENTRY) ; 

        Hashtable attributes = null ; 
        try {
            Connection conn = ConnectionFactory.getConnection() ; 

            String sql = "SELECT mbeanObject " ; 
            sql += "FROM MBean " ; 
            sql += "WHERE mbeanName = '" + domain + ":" + keys.toString() + "'" ;
            Log.write("SQL_VERBOSE executing: " + sql, true, Log.SQL_VERBOSE) ; 

            List results = (List)GenericDBProxy.executeQuery(sql, DatabaseProxy.LIST) ;           
            if ( results.size() != 1 ) {
                throw new Exception() ; 
            }
            attributes = (Hashtable)((TreeMap)results.get(0)).get("mbeanObject") ; 

        } catch (Exception e) {
            e.printStackTrace() ; 
            Log.write(e) ; 
        }
        Log.write("EXIT", true, Log.METHOD_EXIT) ; 
        return attributes ; 
    } 

/*
 * Properties
 */

	/**  
	 * CVS info about this class and its current version
	 */
	public static final String about = "$Id: DBPersistenceManager.java,v 1.1 2001/12/03 12:45:58 tstephen Exp $" ;

}
