// Package declaration 
package org.thestephensons.db ; 

// Java imports 
import java.io.File ; 
import java.io.FileOutputStream ; 
import java.sql.Connection ; 
import java.sql.DatabaseMetaData ; 
import java.sql.ResultSet ; 
import java.util.ArrayList ; 
import java.util.Iterator ; 
import java.util.StringTokenizer ; 
import java.util.TreeMap ; 

// My Imports 
import org.enableit.util.Log ; 
import org.thestephensons.xml.* ; 

/** 
 * 
 * ASA interpretation of DatabaseMetaData object 
 * SCHEMAS = users and groups
 */
public class DBToXmlTransformer extends org.apache.tools.ant.Task
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	public DBToXmlTransformer() 
	{

    } 

    /** 
     * Sets the target (output) directory 
     */
    public void setTargetDir( String targetDir ) 
    { 
        this.targetDir = targetDir ; 
    } 

    /**  
     * Set the table name 
     */
    public void setTableName( String tableName ) 
    { 
        this.tableName = tableName ; 
    } 
    
    /**  
     * Set the JDBC catalogue name (sybase and moicrosoft users are more likely 
     * to know this as database name) 
     */
    public void setCatalog( String catalog ) 
    { 
        this.catalog = catalog ; 
    } 

    /** 
     * Sets the JDBC driver to use in making a connection 
     */
    public void setDriver( String driver ) 
    { 
        this.driver = driver ; 
    } 

    /** 
     * Sets the JDBC database to use in making a connection 
     */
    public void setDatabase( String database ) 
    { 
        this.database = database ; 
    }     
    

    /** 
     * Sets the JDBC password to use in making a connection 
     */
    public void setPassword( String password ) 
    { 
        this.password = password ; 
    } 
    
        
    /** 
     * Sets the JDBC port to use in making a connection 
     */
    public void setPort( String port ) 
    { 
        this.port = port ; 
    } 
    
    /** 
     * Sets the JDBC protocol to use in making a connection 
     */
    public void setProtocol( String protocol ) 
    { 
        this.protocol = protocol ; 
    } 

    /** 
     * Sets the JDBC server to use in making a connection 
     */
    public void setServer( String server ) 
    { 
        this.server = server ; 
    } 

    /** 
     * Sets the JDBC user to use in making a connection 
     */
    public void setUser( String user ) 
    { 
        this.user = user ; 
    } 

    public void execute() 
    { 
        try {
            Connection conn = ConnectionFactory.getConnection(driver, 
                                                              database,
                                                              server, 
                                                              port,   
                                                              protocol,
                                                              user,   
                                                              password );
            if (conn==null) {
                Log.write("Null connection", true, Log.VERBOSE) ; 
                throw new DBException("Null connection") ;
            }
            DatabaseMetaData dmd = conn.getMetaData() ; 

            ResultSet catalogs = dmd.getCatalogs() ; 
            ArrayList catList = (ArrayList)DBUtils.convertResultToList(catalogs) ; 
            Log.write(catList.toString());

            
            ResultSet schemas = dmd.getSchemas() ; 
            ArrayList schList = (ArrayList)DBUtils.convertResultToList(schemas) ; 
            Log.write(schList.toString());

            // Capture Primary Key Name
            ResultSet pk = dmd.getPrimaryKeys(catalog, "DBA", tableName) ; 
            ArrayList pkList = (ArrayList)DBUtils.convertResultToList(pk) ; 
            if (pkList.size()==0) {
                String msg = "STOP: No primary key was found!" ; 
                Log.write(msg, true, Log.VERBOSE) ; 
                throw new DBException(msg) ; 
            }
            TreeMap pkCol = (TreeMap)pkList.get(0); 
            Log.write(pkList.toString(), true, Log.VERBOSE) ; 

            // All columns data 
            ResultSet columns = dmd.getColumns(catalog, "DBA", tableName, null) ; 
            ArrayList colList = (ArrayList)DBUtils.convertResultToList(columns) ; 
            ArrayList pks = new ArrayList() ; 
            pks.add( ((TreeMap)colList.get(0)).get("COLUMN_NAME") ) ; 
            
            // Construct an xml doc 
            XmlRootElement doc = new XmlRootElement(APP) ;
            /*
             * The Java portion 
             */
            XmlTreeElement javaTag = new XmlTreeElement(JAVA) ; 
            javaTag.add( new XmlLeafElement(JAVA_PACKAGE_DIR, getPackageDir(JAVA_PACKAGE_VALUE)) ) ; 
            javaTag.add( new XmlLeafElement(JAVA_PACKAGE_NAME, JAVA_PACKAGE_VALUE) ) ; 
            
            // add the app document
            doc.add(javaTag) ; 
    
            /* 
             * The table portion 
             */
            XmlTreeElement tableTag = new XmlTreeElement(TABLE) ; 
            tableTag.add(new XmlLeafElement(NAME, tableName)) ; 
            // For each list element (represents a table column) ...
            XmlTreeElement column = null ; 
            for ( Iterator i=colList.iterator() ; i.hasNext() ; ) {
                TreeMap tableCol = (TreeMap)i.next() ;                
                // ... for each column construct column element
                column = new XmlTreeElement(COLUMN) ; 
                
                // get column name 
                String colName = (String)tableCol.get("COLUMN_NAME") ; 
                column.add( new XmlLeafElement(COL_NAME, colName) ) ; 

                // get column datatype 
                //System.out.println(tableCol.get("DATA_TYPE").getClass().getName()) ; 
                int type = ((Integer)tableCol.get("DATA_TYPE")).intValue() ; 
                int scale = ((Integer)tableCol.get("COLUMN_SIZE")).intValue() ; 
                int precision = ((Integer)tableCol.get("DECIMAL_DIGITS")).intValue() ; 
                String colType = new SqlType(type, scale, precision).toString() ;
                column.add( new XmlLeafElement(COL_TYPE, colType) ) ; 

                // get column null / not null
                // assume the worst (includes don't know case) 
                boolean isNullable = false ; 
                System.out.println(tableCol.get("IS_NULLABLE").toString()) ; 
                if ( ((String)tableCol.get("IS_NULLABLE")).equals("YES") ) {  
                    isNullable = true ; 
                }
                column.add( new XmlLeafElement(NULLABLE, String.valueOf(isNullable)) ) ;  

                // determine if column is primary key 
                if ( pks.contains(colName) ) {
                    column.add( new XmlLeafElement(PK, "true") ) ;                  
                }

                tableTag.add(column) ; 
            }
            doc.add(tableTag); 

            String targetFile = new String() ; 
            if ( targetDir != null ) {
                targetFile += targetDir + File.separator ; 
            }
            targetFile += tableName + ".xml" ;
            FileOutputStream fos = new FileOutputStream( targetFile ) ;  
            fos.write(doc.toString().getBytes()) ; 
            fos.close() ; 

        } catch (Exception e) {
            // Things to catch : array out of bounds from no column results found
            Log.write(e.getMessage()) ; 
            if (e.getMessage()==null || e.getMessage().length()==0) {
                e.printStackTrace(System.err) ;
            }         
        }    
    }

/*
 * Methods
 */
    /** 
     * 
     * 
     * @param args 
     * args[0] - tableName to turn into XML representation
     */
    public static void main( String[] args ) 
    { 
        
        DBToXmlTransformer transformer = new DBToXmlTransformer() ; 
        transformer.setTableName( args[0] ) ; 
        transformer.execute() ; 
    } 

    /** 
     * Returns the directory name classes should be stored in based on the 
     * Java package name received
     */
    private String getPackageDir( String javaPackage ) 
    { 
        StringBuffer dir = new StringBuffer() ; 
        StringTokenizer st = new StringTokenizer(javaPackage, ".");
        while (st.hasMoreTokens()) {
            dir.append(st.nextToken()) ; 
            dir.append("/") ; 
        }
 
        // remove final '/' and return 
        return dir.substring(0, dir.length()-1) ; 
    }

/* 
 * Properties
 */
    private String tableName ; 
    private String catalog ; 
    private String targetDir ; 

/*
 * Constant Properties
 */
   
/* 
 * DB Connection stuff 
 */
    private String driver = "" ; 
    private String database = "" ; 
    private String server = "" ; 
    private String port = "" ; 
    private String protocol = "" ; 
    private String user = "" ; 
    private String password = "" ; 

/* 
 * Application specifics
 */
    public static final String JAVA_PACKAGE_DIR = "package-dir" ;     
    public static final String JAVA_PACKAGE_NAME = "package-name" ;     
    public static final String JAVA_PACKAGE_VALUE = "org.thestephensons.webservices" ;     

/* 
 * Document Tags 
 */
    public static final String APP = "app" ; 
    public static final String JAVA = "java" ; 
    public static final String TABLE = "table" ; 
    public static final String NAME = "name" ; 
    public static final String COLUMN = "column" ; 
    public static final String COL_NAME = "col-name" ; 
    public static final String COL_TYPE = "col-type" ; 
    public static final String NULLABLE = "null" ; 
    public static final String PK = "primary-key" ; 

}

