    
package org.thestephensons.db ;

/*
 * PROJECT			: Database integration library
 *
 * COPYRIGHT        : Copyright (C) 2000 Tim Stephenson
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: GenericDBProxy.java,v $
 * Revision 1.2  2001/12/03 12:42:31  tstephen
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.1.1.1  2001/04/23 05:27:12  tstephen
 * finally get under source control
 *
 */

// Java Imports
import java.util.*;
import java.sql.*;

// Sybase Jaguar imports
//import com.sybase.jaguar.sql.*;
//import com.sybase.jaguar.server.*;
//import com.sybase.jaguar.jcm.*;
//import com.sybase.jaguar.util.*;
//import com.sybase.CORBA.jdbc11.*;
//import TabularResults.ResultSet ;

// My imports
import org.enableit.util.Log;

/**
 * <p>Proxy class to wrap any java application database access in a simple form</p>
 *
 * <p>This has not been encountered personally, but it is worth being aware of
 * Bug ID: 4282044
 * java:jdbc_odbc_bridge, Callable Statement that does not return result set throws exception invalid proc, State: In progress, bug, Reported: Oct 18,
 * 1999, Release reported against: 1.2.2
 * http://developer.java.sun.com/developer/bugParade/bugs/4282044.html - size 17.1K - Bug Reports</p>
 * 
 * @version __VERSION__
 * 
 * @author tim_stephenson@hotmail.com 
 * 
 */
public class GenericDBProxy extends DatabaseProxy
{

    /**
     * Execute the supplied SQL update query against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sql
	 * The SQL update to execute
	 *
	 * @return int
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(String sql) throws DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}

        int rowsAffected    = 0;

        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
			rowsAffected=executeUpdate(conn, sql);
		} catch (DBException dbE){
			Log.write(dbE.getMessage());
			throw dbE;
		} catch (Exception e) {
			Log.write(e.getMessage(), true, Log.SQL_VERBOSE);
			throw new DBException(e.getMessage());
		} finally {
		}

        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return rowsAffected;
    }

    /**
     * Execute the supplied SQL query against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sql
	 * The SQL query to execute
	 *
	 * @param returnType
	 * int defining the return object types, enumerated on DatabaseProxy
	 *
	 * @return Object
	 * The SQL results as the object type requested
	 *
	 * @throws DBException
	 * If execution of the query failed.
     */
    public static Object executeQuery(String sql, int returnType) 
    throws DBException
    {
		Log.write("ENTRY: sql: " + sql, true, Log.METHOD_ENTRY);

		Object result     = null;

        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
            result=executeQuery(conn, sql, returnType);
        	conn.close() ; 
		} catch (DBException dbe) {
            // already logged, just rethrow
			throw dbe ;
		} catch (Exception e) {
			Log.write(e.getMessage(), true, Log.SQL_VERBOSE);
			throw new DBException("ERROR: " + e.getMessage());
		} finally {

        }

        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return result;
    }

    /**
     * Execute the supplied stored procedure against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sp
	 * A string containing the stored procedure name
	 *
     * @param parms
     * <code>ArrayList</code> holding parameters required for stored proc
	 *
	 * @return int
	 * The number of rows affected by the stored procedure executed
	 *
	 * @throws DBException
	 * If execution of the query failed
     */
    public static int executeDmlProcedure(String sp, ArrayList parms) 
    throws DBException
    {
		Log.write("ENTRY: sp, parms: " + sp + "," + parms.toString(), true, Log.METHOD_ENTRY);

		int i = 0;

        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
			executeDmlProcedure(conn, sp, parms);
            conn.close() ;
		} catch (DBException dbE){
			Log.write(dbE.getMessage(), true, Log.SQL_VERBOSE);
            throw dbE;
		} catch (Exception e) {
			Log.write(e.getMessage(), true, Log.SQL_VERBOSE);
			throw new DBException(e.getMessage());
		} finally {
		}

        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return i ;

	}

	/**
     * Execute the supplied stored procedure against a database connection
	 * supplied from <code>ConnectionFactory</code>.
	 *
	 * @param sp
	 * A string containing the stored procedure name
	 *
     * @param parms
     * <code>ArrayList</code> holding parameters required for stored proc
	 *
	 * @param returnType
	 * int defining the return object types, enumerated on DatabaseProxy
	 *
	 * @return object
	 * <code>java.lang.Object</code> containing any rows found
     * Object type matches the enumeration in <code>DatabaseProxy</code>
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static java.lang.Object executeQueryProcedure(String sp, List parms, int returnType) 
    throws DBException
    {
		Log.write("ENTRY: sp, parms: " + sp + "," + parms.toString(), true, Log.METHOD_ENTRY);

        Object o = null ; 
        try {
			// get connection, create statement and execute
            Connection conn = ConnectionFactory.getConnection() ;
    		o = executeQueryProcedure(conn, sp, parms, returnType) ;
		} catch (DBException dbE){
			Log.write(dbE.getMessage());
			throw dbE;
		} catch (Exception e) {
			Log.write(e.getMessage(), true, Log.SQL_VERBOSE);
			throw new DBException(e.getMessage());
		} finally {
        }

        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return o ;
	}


    /** 
     * Information on the exact CVS version accessible after compilation
     */
    public static final String about = "$Revision: 1.2 $";

}


